import re

import networkx as nx
import numpy as np
from nltk.corpus import stopwords


class Summarizer:
    def read_article(self, text):
        # Split the text into sentences
        sentences = re.split(r'(?<=[^A-Z].[.?]) +(?=[A-Z])', text)

        # Pre-process the sentences by removing special characters and lowercasing
        processed_sentences = []
        for sentence in sentences:
            processed_sentences.append(re.sub(r'[^\w\s]', '', sentence).lower())

        return processed_sentences

    def sentence_similarity(self, sent1, sent2, stopwords=None):
        if stopwords is None:
            stopwords = []

        # Split the sentences into words and remove stopwords
        sent1 = [word for word in sent1.split() if word not in stopwords]
        sent2 = [word for word in sent2.split() if word not in stopwords]


        if len(sent1) == 0 or len(sent2) == 0:
            # There are no words left after removing stopwords
            return 0

        # Get the vocabulary and create a set for each sentence
        vocabulary = set(sent1).union(set(sent2))
        set1 = set(sent1)
        set2 = set(sent2)

        # Compute the Jaccard distance between the sets
        jaccard_distance = len(set1.intersection(set2)) / len(set1.union(set2))
        return jaccard_distance

    def build_similarity_matrix(self, sentences, stop_words):
        # Create an empty similarity matrix
        similarity_matrix = np.zeros((len(sentences), len(sentences)))

        # For each sentence, compare it to every other sentence and compute similarity
        for idx1 in range(len(sentences)):
            for idx2 in range(len(sentences)):
                if idx1 == idx2:  # ignore if both are same sentences
                    continue
                similarity_matrix[idx1][idx2] = self.sentence_similarity(sentences[idx1], sentences[idx2], stop_words)

        return similarity_matrix

    def generate_summary(self, text, top_n=5):
        stop_words = stopwords.words('english')

        # Step 1 - Read text and split it into sentences
        sentences = self.read_article(text)

        # Step 2 - Generate Similary Matrix across sentences
        sentence_similarity_matrix = self.build_similarity_matrix(sentences, stop_words)

        # Step 3 - Rank sentences in similarity matrix
        sentence_similarity_graph = nx.from_numpy_array(sentence_similarity_matrix)
        scores = nx.pagerank(sentence_similarity_graph)

        # Step 4 - Sort the rank and pick top sentences
        ranked_sentences = sorted(((scores[i], s) for i, s in enumerate(sentences)), reverse=True)

        # Step 5 - Return the top N ranked sentences as the summary
        num_sentences = min(top_n, len(ranked_sentences)) # NOTE: added to prevent internal server error for short articles
        return [ranked_sentences[i][1] for i in range(num_sentences)]
